import { __awaiter, __generator } from "tslib";
import { EventStreamCodec } from "@aws-sdk/eventstream-codec";
import { PassThrough, pipeline, Readable } from "stream";
import { EventSigningStream } from "./EventSigningStream";
var EventStreamPayloadHandler = (function () {
    function EventStreamPayloadHandler(options) {
        this.eventSigner = options.eventSigner;
        this.eventStreamCodec = new EventStreamCodec(options.utf8Encoder, options.utf8Decoder);
    }
    EventStreamPayloadHandler.prototype.handle = function (next, args, context) {
        if (context === void 0) { context = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var request, payload, payloadStream, result, e_1, match, priorSignature, signingStream, _a;
            var _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        request = args.request;
                        payload = request.body;
                        if (!(payload instanceof Readable)) {
                            throw new Error("Eventstream payload must be a Readable stream.");
                        }
                        payloadStream = payload;
                        request.body = new PassThrough({
                            objectMode: true,
                        });
                        _c.label = 1;
                    case 1:
                        _c.trys.push([1, 3, , 4]);
                        return [4, next(args)];
                    case 2:
                        result = _c.sent();
                        return [3, 4];
                    case 3:
                        e_1 = _c.sent();
                        request.body.end();
                        throw e_1;
                    case 4:
                        match = (request.headers["authorization"] || "").match(/Signature=([\w]+)$/);
                        priorSignature = (match || [])[1];
                        _a = EventSigningStream.bind;
                        _b = {
                            priorSignature: priorSignature,
                            eventStreamCodec: this.eventStreamCodec
                        };
                        return [4, this.eventSigner()];
                    case 5:
                        signingStream = new (_a.apply(EventSigningStream, [void 0, (_b.eventSigner = _c.sent(),
                                _b)]))();
                        pipeline(payloadStream, signingStream, request.body, function (err) {
                            if (err) {
                                throw err;
                            }
                        });
                        return [2, result];
                }
            });
        });
    };
    return EventStreamPayloadHandler;
}());
export { EventStreamPayloadHandler };
